@@ -85,7 +85,7 @@ public class Storage implements Closeable
   /** The default piece size. */
   private static final int DEFAULT_PIECE_SIZE = 256*1024;
   /** bigger than this will be rejected */
-  public static final int MAX_PIECE_SIZE = 16*1024*1024;
+  public static final int MAX_PIECE_SIZE = 32*1024*1024;
   /** The maximum number of pieces in a torrent. */
   public static final int MAX_PIECES = 32*1024;
   public static final long MAX_TOTAL_SIZE = MAX_PIECE_SIZE * (long) MAX_PIECES;
@@ -433,9 +433,22 @@ public class Storage implements Closeable
    *  @since 0.9.23
    */
   public long[] remaining() {
+      return remaining2()[0];
+  }
+
+  /**
+   *  For efficiency, calculate remaining bytes for all files at once.
+   *  Remaining bytes is rv[0]. Preview bytes is rv[1].
+   *
+   *  @return number of bytes remaining and number of bytes available for a preview for each file, use indexOf() to get index for a file
+   *  @since 0.9.45
+   */
+  public long[][] remaining2() {
       long[] rv = new long[_torrentFiles.size()];
+      long[] pv = new long[_torrentFiles.size()];
+      long[][] rva = new long[][] { rv, pv };
       if (complete())
-          return rv;
+          return rva;
       long bytes = 0;
       for (int i = 0; i < _torrentFiles.size(); i++) {
           TorrentFile tf = _torrentFiles.get(i);
@@ -443,10 +456,23 @@ public class Storage implements Closeable
           long end = start + tf.length;
           int pc = (int) (bytes / piece_size);
           long rvi = 0;
-          if (!bitfield.get(pc))
-              rvi = Math.min(piece_size - (start % piece_size), tf.length);
+          long pvi = 0;
+          long first = Math.min(piece_size - (start % piece_size), tf.length);
+          if (bitfield.get(pc))
+              pvi = first;
+          else
+              rvi = first;
+          boolean preview = true;
           for (int j = pc + 1; (((long)j) * piece_size) < end && j < pieces; j++) {
-              if (!bitfield.get(j)) {
+              if (bitfield.get(j)) {
+                  if (preview) {
+                      if (((long)(j+1))*piece_size < end)
+                          pvi += piece_size;
+                      else
+                          pvi += end - (((long)j) * piece_size);
+                  }
+              } else {
+                  preview = false;
                   if (((long)(j+1))*piece_size < end)
                       rvi += piece_size;
                   else
@@ -454,9 +480,10 @@ public class Storage implements Closeable
               }
           }
           rv[i] = rvi;
+          pv[i] = pvi;
           bytes += tf.length;
       }
-      return rv;
+      return rva;
   }
 
   /**
