@@ -89,6 +89,8 @@ public class TunnelController implements Logging {
     public static final String PROP_FILTER = "filterDefinition";
     /** @since 0.9.42 */
     public static final String PROP_CONFIG_FILE = "configFile";
+    /** @since 0.9.46 */
+    public static final String PROP_TUN_GZIP = "i2ptunnel.gzip";
 
     /**
      * all of these are @since 0.9.33 (moved from TunnelConfig)
@@ -101,13 +103,13 @@ public class TunnelController implements Logging {
     public static final String PROP_MAX_TOTAL_CONNS_DAY = "i2p.streaming.maxTotalConnsPerDay";
     public static final String PROP_MAX_STREAMS = "i2p.streaming.maxConcurrentStreams";
     public static final String PROP_LIMITS_SET = "i2p.streaming.limitsManuallySet";
-    public static final int DEFAULT_MAX_CONNS_MIN = 15;
-    public static final int DEFAULT_MAX_CONNS_HOUR = 40;
-    public static final int DEFAULT_MAX_CONNS_DAY = 100;
-    public static final int DEFAULT_MAX_TOTAL_CONNS_MIN = 25;
+    public static final int DEFAULT_MAX_CONNS_MIN = 30;
+    public static final int DEFAULT_MAX_CONNS_HOUR = 80;
+    public static final int DEFAULT_MAX_CONNS_DAY = 200;
+    public static final int DEFAULT_MAX_TOTAL_CONNS_MIN = 50;
     public static final int DEFAULT_MAX_TOTAL_CONNS_HOUR = 0;
     public static final int DEFAULT_MAX_TOTAL_CONNS_DAY = 0;
-    public static final int DEFAULT_MAX_STREAMS = 20;
+    public static final int DEFAULT_MAX_STREAMS = 30;
 
     /** @since 0.9.34 */
     public static final String PROP_LIMIT_ACTION = "i2p.streaming.limitAction";
@@ -139,6 +141,9 @@ public class TunnelController implements Logging {
 
     /** @since 0.9.34 */
     private static final String OPT_LIMIT_ACTION = PFX_OPTION + PROP_LIMIT_ACTION;
+    private static final String OPT_I2CP_GZIP = PFX_OPTION + I2PClient.PROP_GZIP;
+
+    private static final String OPT_ENCTYPE = PFX_OPTION + "i2cp.leaseSetEncType";
 
     /** all of these @since 0.9.14 */
     public static final String TYPE_CONNECT = "connectclient";
@@ -836,6 +841,15 @@ public class TunnelController implements Logging {
                     _config.setProperty(OPT_LIMIT_ACTION, "http");
             }
             if (type.equals(TYPE_HTTP_SERVER) || type.equals(TYPE_STREAMR_SERVER)) {
+                String tgzip = _config.getProperty(PROP_TUN_GZIP);
+                if (tgzip == null || Boolean.valueOf(tgzip)) {
+                    // Web server will gzip
+                    // If web server doesn't gzip, I2PTunnelHTTPServer will.
+                    // Streaming will force gzip on first packet for header compression,
+                    // regardless of this setting
+                    if (!_config.containsKey(OPT_I2CP_GZIP))
+                        _config.setProperty(OPT_I2CP_GZIP, "false");
+                }
                 if (!_config.containsKey(OPT_BUNDLE_REPLY))
                     _config.setProperty(OPT_BUNDLE_REPLY, "false");
             } else if (!isClient(type)) {
@@ -849,12 +863,12 @@ public class TunnelController implements Logging {
                 if (!_config.containsKey(OPT_LOW_TAGS))
                     _config.setProperty(OPT_LOW_TAGS, "14");
             }
-            // same default logic as in EditBean.getSigType()
+            // same default logic as in EditBean.getSigType() and GeneralHelper.getSigType()
             if (!isClient(type) ||
                 type.equals(TYPE_IRC_CLIENT) || type.equals(TYPE_STD_CLIENT) ||
-                type.equals(TYPE_SOCKS) ||
+                type.equals(TYPE_SOCKS) || type.equals(TYPE_CONNECT) ||
                 type.equals(TYPE_SOCKS_IRC) || type.equals(TYPE_STREAMR_CLIENT) ||
-                (type.equals(TYPE_HTTP_CLIENT) && Boolean.valueOf(getSharedClient()))) {
+                type.equals(TYPE_HTTP_CLIENT)) {
                 if (!_config.containsKey(OPT_SIG_TYPE))
                     _config.setProperty(OPT_SIG_TYPE, PREFERRED_SIGTYPE.name());
             }
@@ -888,6 +902,12 @@ public class TunnelController implements Logging {
                     }
                 }
             }
+            if (isClient(type) &&
+                (type.equals(TYPE_HTTP_CLIENT) || Boolean.valueOf(_config.getProperty(PROP_SHARED, "false")))) {
+                // migration: HTTP proxy and shared clients default to both
+                if (!_config.containsKey(OPT_ENCTYPE))
+                    _config.setProperty(OPT_ENCTYPE, "4,0");
+            }
         }
 
         // tell i2ptunnel, who will tell the TunnelTask, who will tell the SocketManager
@@ -999,6 +1019,7 @@ public class TunnelController implements Logging {
      *  Note that a streamr server is a UI and I2P server but a client on the localhost side.
      *
      *  @since 0.9.17 moved from IndexBean
+     *  @return false if type == null
      */
     public static boolean isClient(String type) {
         return TYPE_STD_CLIENT.equals(type) ||
@@ -1044,8 +1065,14 @@ public class TunnelController implements Logging {
     public String getListenPort() { return _config.getProperty(PROP_LISTEN_PORT); }
     public String getTargetDestination() { return _config.getProperty(PROP_DEST); }
     public String getProxyList() { return _config.getProperty(PROP_PROXIES); }
-    /** default true */
-    public String getSharedClient() { return _config.getProperty(PROP_SHARED, "true"); }
+
+    /** default true for clients, always false for servers */
+    public String getSharedClient() {
+         if (!isClient())
+             return "false";
+         return _config.getProperty(PROP_SHARED, "true");
+    }
+
     /** default true */
     public boolean getStartOnLoad() { return Boolean.parseBoolean(_config.getProperty(PROP_START, "true")); }
     public boolean getPersistentClientKey() { return Boolean.parseBoolean(_config.getProperty(OPT_PERSISTENT)); }
