@@ -22,6 +22,7 @@ import java.util.Set;
 import java.util.SortedSet;
 import java.util.StringTokenizer;
 import java.util.concurrent.LinkedBlockingQueue;
+import javax.servlet.ServletRequest;
 
 import net.i2p.I2PAppContext;
 import net.i2p.app.ClientApp;
@@ -129,8 +130,8 @@ public class RouterConsoleRunner implements RouterApp {
     public static final String ROUTERCONSOLE = "routerconsole";
     public static final String PREFIX = "webapps.";
     public static final String ENABLED = ".startOnLoad";
-    private static final String PROP_KEYSTORE_PASSWORD = "routerconsole.keystorePassword";
-    private static final String PROP_KEY_PASSWORD = "routerconsole.keyPassword";
+    public static final String PROP_KEYSTORE_PASSWORD = "routerconsole.keystorePassword";
+    public static final String PROP_KEY_PASSWORD = "routerconsole.keyPassword";
     public static final int DEFAULT_LISTEN_PORT = PortMapper.DEFAULT_CONSOLE_PORT;
     private static final String DEFAULT_WEBAPPS_DIR = "./webapps/";
     private static final String USAGE = "Bad RouterConsoleRunner arguments, check clientApp.0.args in your clients.config file! " +
@@ -345,8 +346,9 @@ public class RouterConsoleRunner implements RouterApp {
     /** @since 0.9.17 */
     private void checkJavaVersion() {
         boolean noJava8 = !SystemVersion.isJava8();
-        boolean noPack200 = (PluginStarter.pluginsEnabled(_context) || !NewsHelper.isUpdateDisabled(_context)) &&
-                            !FileUtil.isPack200Supported();
+        //boolean noPack200 = (PluginStarter.pluginsEnabled(_context) || !NewsHelper.isUpdateDisabled(_context)) &&
+        //                    !FileUtil.isPack200Supported();
+        boolean noPack200 = false;
         boolean openARM = SystemVersion.isARM() && SystemVersion.isOpenJDK() && !SystemVersion.isJava9();
         boolean isZero = SystemVersion.isZeroVM();
         boolean isJava11 = false; // SystemVersion.isJava11();
@@ -359,17 +361,20 @@ public class RouterConsoleRunner implements RouterApp {
             log.logAlways(net.i2p.util.Log.WARN, s);
             System.out.println("Warning: " + s);
             if (noJava8) {
-                s = "Java 8 or higher will be required in a future release, please upgrade Java";
+                s = "Java 8 or higher is required, please upgrade Java";
                 log.logAlways(net.i2p.util.Log.WARN, s);
                 System.out.println("Warning: " + s);
             }
             if (noPack200) {
-                s = "Pack200 is required for plugins and automatic updates, please upgrade Java";
+                if (SystemVersion.isJava(14))
+                    s = "Pack200 is required for some plugins, please consider downgrading Java to 13 or lower";
+                else
+                    s = "Pack200 is required for some plugins, please consider upgrading Java";
                 log.logAlways(net.i2p.util.Log.WARN, s);
                 System.out.println("Warning: " + s);
             }
             if (openARM) {
-                s = "OpenJDK 7/8 are not recommended for ARM. Use OpenJDK 9 (or higher) or Oracle Java 8 (or higher)";
+                s = "OpenJDK 8 is not recommended for ARM. Use OpenJDK 9 (or higher) or Oracle Java 8 (or higher)";
                 log.logAlways(net.i2p.util.Log.WARN, s);
                 System.out.println("Warning: " + s);
             }
@@ -882,8 +887,9 @@ public class RouterConsoleRunner implements RouterApp {
             ConfigServiceHandler.registerSignalHandler(_context);
 
             if (_mgr != null &&
-                _context.getBooleanProperty(HelperBase.PROP_ADVANCED) &&
-                _context.getProperty(Analysis.PROP_FREQUENCY, 0L) > 0) {
+                //_context.getBooleanProperty(HelperBase.PROP_ADVANCED) &&
+                !SystemVersion.isSlow() &&
+                _context.getProperty(Analysis.PROP_FREQUENCY, Analysis.DEFAULT_FREQUENCY) > 0) {
                 // registers and starts itself
                 Analysis.getInstance(_context);
             }
@@ -1080,8 +1086,8 @@ public class RouterConsoleRunner implements RouterApp {
         }
 
         @Override
-        public UserIdentity login(String username, Object credentials) {
-            UserIdentity rv = super.login(username, credentials);
+        public UserIdentity login(String username, Object credentials, ServletRequest request) {
+            UserIdentity rv = super.login(username, credentials, request);
             if (rv == null)
                 //_log.logAlways(net.i2p.util.Log.WARN, "Console authentication failed, webapp: " + _webapp + ", user: " + username);
                 _log.logAlways(net.i2p.util.Log.WARN, "Console authentication failed, user: " + username);
