@@ -8,6 +8,7 @@ package net.i2p.router.web.helpers;
  *
  */
 
+import java.io.File;
 import java.io.IOException;
 import java.io.Serializable;
 import java.io.Writer;
@@ -42,6 +43,7 @@ import net.i2p.router.TunnelPoolSettings;
 import net.i2p.router.util.HashDistance;   // debug
 import net.i2p.router.networkdb.kademlia.FloodfillNetworkDatabaseFacade;
 import static net.i2p.router.sybil.Util.biLog2;
+import net.i2p.router.transport.GeoIP;
 import net.i2p.router.web.HelperBase;
 import net.i2p.router.web.Messages;
 import net.i2p.router.web.WebAppStarter;
@@ -732,10 +734,11 @@ class NetDbRenderer {
             }
             buf.append("</div>");
         }
-        long end = System.currentTimeMillis();
-        if (log.shouldWarn())
+        if (log.shouldWarn()) {
+            long end = System.currentTimeMillis();
             log.warn("part 1 took " + (end - start));
-        start = end;
+            start = end;
+        }
 
      //
      // don't bother to reindent
@@ -763,10 +766,11 @@ class NetDbRenderer {
         buf.append("</td><td style=\"vertical-align: top;\">");
         out.write(buf.toString());
         buf.setLength(0);
-        end = System.currentTimeMillis();
-        if (log.shouldWarn())
+        if (log.shouldWarn()) {
+            long end = System.currentTimeMillis();
             log.warn("part 2 took " + (end - start));
-        start = end;
+            start = end;
+        }
 
         // transports table
         boolean showTransports = _context.getBooleanProperty(HelperBase.PROP_ADVANCED);
@@ -784,10 +788,11 @@ class NetDbRenderer {
             buf.append("</td><td style=\"vertical-align: top;\">");
             out.write(buf.toString());
             buf.setLength(0);
-            end = System.currentTimeMillis();
-            if (log.shouldWarn())
+            if (log.shouldWarn()) {
+                long end = System.currentTimeMillis();
                 log.warn("part 3 took " + (end - start));
-            start = end;
+                start = end;
+            }
         }
 
         // country table
@@ -804,14 +809,24 @@ class NetDbRenderer {
                 buf.append(getTranslatedCountry(country));
                 buf.append("</a></td><td align=\"center\">").append(num).append("</td></tr>\n");
             }
+            // https://db-ip.com/db/download/ip-to-country-lite
+            String geoDir = _context.getProperty(GeoIP.PROP_GEOIP_DIR, GeoIP.GEOIP_DIR_DEFAULT);
+            File geoFile = new File(geoDir);
+            if (!geoFile.isAbsolute())
+                geoFile = new File(_context.getBaseDir(), geoDir);
+            geoFile = new File(geoFile, GeoIP.GEOIP2_FILE_DEFAULT);
+            if (geoFile.exists()) {
+                // we'll assume we are using it, ignore case where Debian file is newer
+                buf.append("<tr><td colspan=\"2\"><font size=\"-2\"><a href=\"https://db-ip.com/\">IP Geolocation by DB-IP</a></font></td></tr>");
+            }
             buf.append("</table>\n");
         }
 
         buf.append("</td></tr></table>");
-        end = System.currentTimeMillis();
-        if (log.shouldWarn())
+        if (log.shouldWarn()) {
+            long end = System.currentTimeMillis();
             log.warn("part 4 took " + (end - start));
-        start = end;
+        }
 
      //
      // don't bother to reindent
@@ -879,12 +894,14 @@ class NetDbRenderer {
 
     private void renderRouterInfo(StringBuilder buf, RouterInfo info, boolean isUs, boolean full) {
         String hash = info.getIdentity().getHash().toBase64();
-        buf.append("<table class=\"netdbentry\">")
-           .append("<tr><th colspan=\"2\"><a name=\"").append(hash.substring(0, 6)).append("\" ></a>");
+        buf.append("<table class=\"netdbentry\">" +
+                   "<tr id=\"").append(hash.substring(0, 6)).append("\"><th colspan=\"2\"");
         if (isUs) {
-            buf.append("<a name=\"our-info\" ></a><b>" + _t("Our info") + ":</b>&nbsp;<code>").append(hash).append("</code></th><th>");
+            buf.append(" id=\"our-info\"><b>").append(_t("Our Router Identity")).append(":</b> <code>")
+               .append(hash).append("</code></th><th>");
         } else {
-            buf.append("<b>" + _t("Peer info for") + ":</b>&nbsp;<code>").append(hash).append("</code></th><th>");
+            buf.append("><b>").append(_t("Router")).append(":</b> <code>")
+               .append(hash).append("</code></th><th>");
             String country = _context.commSystem().getCountry(info.getIdentity().getHash());
             if (country != null) {
                 buf.append("<a href=\"/netdb?c=").append(country).append("\">");
@@ -904,26 +921,28 @@ class NetDbRenderer {
             buf.append("<td><b>").append(_t("Hidden")).append(", ").append(_t("Updated")).append(":</b></td>")
                .append("<td colspan=\"2\"><span class=\"netdb_info\">")
                .append(_t("{0} ago", DataHelper.formatDuration2(age)))
-               .append("</span>&nbsp;&nbsp;");
+               .append("</span>");
         } else if (age > 0) {
             buf.append("<td><b>").append(_t("Published")).append(":</b></td>")
                .append("<td colspan=\"2\"><span class=\"netdb_info\">")
                .append(_t("{0} ago", DataHelper.formatDuration2(age)))
-               .append("</span>&nbsp;&nbsp;");
+               .append("</span>");
         } else {
             // shouldnt happen
             buf.append("<td><b>").append(_t("Published")).append("</td><td colspan=\"2\">:</b> in ")
-               .append(DataHelper.formatDuration2(0-age)).append("<span class=\"netdb_info\">???</span>&nbsp;&nbsp;");
+               .append(DataHelper.formatDuration2(0-age)).append("<span class=\"netdb_info\">???</span>");
+        }
+        if (full) {
+            buf.append("</td></tr><tr><td><b>").append(_t("Signing Key")).append(":</b></td><td colspan=\"2\">")
+               .append(info.getIdentity().getSigningPublicKey().getType().toString());
         }
-        buf.append("<b>").append(_t("Signing Key")).append(":</b> ")
-           .append(info.getIdentity().getSigningPublicKey().getType().toString());
         buf.append("</td></tr>\n<tr>")
-           .append("<td><b>" + _t("Addresses") + ":</b></td>")
-           .append("<td colspan=\"2\" class=\"netdb_addresses\">");
+           .append("<td><b>").append(_t("Addresses")).append(":</b></td><td colspan=\"2\"");
         Collection<RouterAddress> addrs = info.getAddresses();
         if (addrs.isEmpty()) {
-            buf.append(_t("none"));
+            buf.append('>').append(_t("none"));
         } else {
+            buf.append(" class=\"netdb_addresses\">");
             if (addrs.size() > 1) {
                 // addrs is unmodifiable
                 List<RouterAddress> laddrs = new ArrayList<RouterAddress>(addrs);
