@@ -93,7 +93,7 @@ class ProfileOrganizerRenderer {
                    buf.append("<tr>");
                    buf.append("<th>").append(_t("Peer")).append("</th>");
                    buf.append("<th>").append(_t("Groups")).append("</th>");
-                   buf.append("<th>").append(_t("Caps")).append("</th>");
+                   buf.append("<th>").append("Caps").append("</th>");
                    buf.append("<th>").append(_t("Version")).append("</th>");
                    buf.append("<th>").append(_t("Speed")).append("</th>");
                    buf.append("<th>").append(_t("Capacity")).append("</th>");
@@ -221,7 +221,7 @@ class ProfileOrganizerRenderer {
         buf.append("<div class=\"widescroll\"><table id=\"floodfills\">");
         buf.append("<tr class=\"smallhead\">");
         buf.append("<th>").append(_t("Peer")).append("</th>");
-        buf.append("<th>").append(_t("Caps")).append("</th>");
+        buf.append("<th>").append("Caps").append("</th>");
         buf.append("<th>").append(_t("Integ. Value")).append("</th>");
         buf.append("<th>").append(_t("Last Heard About")).append("</th>");
         buf.append("<th>").append(_t("Last Heard From")).append("</th>");
@@ -297,10 +297,10 @@ class ProfileOrganizerRenderer {
         buf.append("<h3 class=\"tabletitle\">").append(_t("Definitions")).append("</h3>\n")
            .append("<table id=\"profile_defs\"><tbody>");
         buf.append("<tr><td><b>")
-           .append(_t("groups")).append(":</b></td><td>").append(_t("as determined by the profile organizer"))
+           .append(_t("Groups")).append(":</b></td><td>").append(_t("as determined by the profile organizer"))
            .append("</td></tr>");
         buf.append("<tr><td><b>")
-           .append(_t("caps")).append(":</b></td><td>").append(_t("capabilities in the netDb, not used to determine profiles"))
+           .append("Caps").append(":</b></td><td>").append(_t("capabilities in the netDb, not used to determine profiles"))
            .append("</td></tr>");
         buf.append("<tr id=\"capabilities_key\"><td colspan=\"2\"><table><tbody>");
         buf.append("<tr><td>&nbsp;</td>")
