@@ -117,15 +117,15 @@ class PacketQueue implements SendMessageStatusListener, Closeable {
             // this should not block!
             begin = _context.clock().now();
             long expires = 0;
-            Connection.ResendPacketEvent rpe = (Connection.ResendPacketEvent) packet.getResendEvent();
-            if (rpe != null) {
+            int pktTimeout = packet.getTimeout();
+            if (pktTimeout > 0) {
                 // we want the router to expire it a little before we do,
                 // so if we retransmit it will use a new tunnel/lease combo
                 // If we are really close to the timeout already,
                 // give this packet a chance to be sent,
                 // but it's likely to be dropped on the router side if we're
                 // running this far behind.
-                expires = Math.max(rpe.getNextSendTime() - I2CP_EXPIRATION_ADJUST, begin + 25);
+                expires = Math.max(begin + pktTimeout - I2CP_EXPIRATION_ADJUST, begin + 25);
             }
             SendMessageOptions options = new SendMessageOptions();
             if (expires > 0)
@@ -344,6 +344,16 @@ class PacketQueue implements SendMessageStatusListener, Closeable {
                 }
                 break;
 
+            case MessageStatusMessage.STATUS_SEND_FAILURE_META_LEASESET:
+                // TODO
+                _messageStatusMap.remove(id);
+                IOException ioe = new I2PSocketException(status);
+                con.getOutputStream().streamErrorOccurred(ioe);
+                con.getInputStream().streamErrorOccurred(ioe);
+                con.setConnectionError(ioe.getLocalizedMessage());
+                con.disconnect(false);
+                break;
+
             case MessageStatusMessage.STATUS_SEND_BEST_EFFORT_SUCCESS:
             case MessageStatusMessage.STATUS_SEND_GUARANTEED_SUCCESS:
             case MessageStatusMessage.STATUS_SEND_SUCCESS_LOCAL:
