@@ -42,6 +42,7 @@ import java.util.Properties;
 import java.util.TimeZone;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.regex.Pattern;
+import java.util.zip.CRC32;
 import java.util.zip.Deflater;
 
 import net.i2p.I2PAppContext;
@@ -806,6 +807,37 @@ public class DataHelper {
             throw new IllegalArgumentException("fromLong got a negative? " + rv + ": offset="+ offset +" numBytes="+numBytes);
         return rv;
     }
+
+    /**
+     * Big endian.
+     * Same as fromLong(src, offset, 8) but allows negative result
+     *
+     * @throws ArrayIndexOutOfBoundsException
+     * @since 0.9.47 moved from NTCP2Payload
+     */
+    public static long fromLong8(byte src[], int offset) {
+        long rv = 0;
+        int limit = offset + 8;
+        for (int i = offset; i < limit; i++) {
+            rv <<= 8;
+            rv |= src[i] & 0xFF;
+        }
+        return rv;
+    }
+    
+    /**
+     * Big endian.
+     * Same as toLong(target, offset, 8, value) but allows negative value
+     *
+     * @throws ArrayIndexOutOfBoundsException
+     * @since 0.9.47 moved from NTCP2Payload
+     */
+    public static void toLong8(byte target[], int offset, long value) {
+        for (int i = offset + 7; i >= offset; i--) {
+            target[i] = (byte) value;
+            value >>= 8;
+        }
+    }
     
     /** Read in a date from the stream as specified by the I2P data structure spec.
      * A date is an 8 byte unsigned integer in network byte order specifying the number of
@@ -1380,25 +1412,25 @@ public class DataHelper {
             // {0,number,####} prevents 1234 from being output as 1,234 in the English locale.
             // If you want the digit separator in your locale, translate as {0}.
             // alternates: msec, msecs
-            t = ngettext("1 ms", "{0,number,####} ms", (int) ms);
+            t = ngettext("{0,number,####} ms", "{0,number,####} ms", (int) ms);
         } else if (ams < 2 * 60 * 1000) {
             // seconds
             // alternates: secs, sec. 'seconds' is probably too long.
-            t = ngettext("1 sec", "{0} sec", (int) (ms / 1000));
+            t = ngettext("{0} sec", "{0} sec", (int) (ms / 1000));
         } else if (ams < 120 * 60 * 1000) {
             // minutes
             // alternates: mins, min. 'minutes' is probably too long.
-            t = ngettext("1 min", "{0} min", (int) (ms / (60 * 1000)));
+            t = ngettext("{0} min", "{0} min", (int) (ms / (60 * 1000)));
         } else if (ams < 2 * 24 * 60 * 60 * 1000) {
             // hours
             // alternates: hrs, hr., hrs.
-            t = ngettext("1 hour", "{0} hours", (int) (ms / (60 * 60 * 1000)));
+            t = ngettext("{0} hour", "{0} hours", (int) (ms / (60 * 60 * 1000)));
         } else if (ams < 3L * 365 * 24 * 60 * 60 * 1000) {
             // days
-            t = ngettext("1 day", "{0} days", (int) (ms / (24 * 60 * 60 * 1000)));
+            t = ngettext("{0} day", "{0} days", (int) (ms / (24 * 60 * 60 * 1000)));
         } else if (ams < 1000L * 365 * 24 * 60 * 60 * 1000) {
             // years
-            t = ngettext("1 year", "{0} years", (int) (ms / (365L * 24 * 60 * 60 * 1000)));
+            t = ngettext("{0} year", "{0} years", (int) (ms / (365L * 24 * 60 * 60 * 1000)));
         } else {
             return _t("n/a");
         }
@@ -1429,23 +1461,23 @@ public class DataHelper {
         if (adms < 0.000000001d) {
             return "0";
         } else if (adms < 0.001d) {
-            t = ngettext("1 ns", "{0,number,###} ns", (int) Math.round(ms * 1000000d));
+            t = ngettext("{0,number,####} ns", "{0,number,###} ns", (int) Math.round(ms * 1000000d));
         } else if (adms < 1.0d) {
-            t = ngettext("1 μs", "{0,number,###} μs", (int) Math.round(ms * 1000d));
+            t = ngettext("{0,number,####} μs", "{0,number,###} μs", (int) Math.round(ms * 1000d));
         } else if (ams < 3 * 1000) {
-            t = ngettext("1 ms", "{0,number,####} ms", (int) Math.round(ms));
+            t = ngettext("{0,number,####} ms", "{0,number,####} ms", (int) Math.round(ms));
         } else if (ams < 2 * 60 * 1000) {
-            t = ngettext("1 sec", "{0} sec", (int) (ms / 1000));
+            t = ngettext("{0} sec", "{0} sec", (int) (ms / 1000));
         } else if (ams < 120 * 60 * 1000) {
-            t = ngettext("1 min", "{0} min", (int) (ms / (60 * 1000)));
+            t = ngettext("{0} min", "{0} min", (int) (ms / (60 * 1000)));
         } else if (ams < 2 * 24 * 60 * 60 * 1000) {
-            t = ngettext("1 hour", "{0} hours", (int) (ms / (60 * 60 * 1000)));
+            t = ngettext("{0} hour", "{0} hours", (int) (ms / (60 * 60 * 1000)));
         } else if (ams < 3L * 365 * 24 * 60 * 60 * 1000) {
             // days
-            t = ngettext("1 day", "{0} days", (int) (ms / (24 * 60 * 60 * 1000)));
+            t = ngettext("{0} day", "{0} days", (int) (ms / (24 * 60 * 60 * 1000)));
         } else if (ams < 1000L * 365 * 24 * 60 * 60 * 1000) {
             // years
-            t = ngettext("1 year", "{0} years", (int) (ms / (365L * 24 * 60 * 60 * 1000)));
+            t = ngettext("{0} year", "{0} years", (int) (ms / (365L * 24 * 60 * 60 * 1000)));
         } else {
             return _t("n/a");
         }
@@ -1454,7 +1486,7 @@ public class DataHelper {
         return t.replace(" ", "&nbsp;");
     }
     
-    private static final String BUNDLE_NAME = "net.i2p.router.web.messages";
+    private static final String BUNDLE_NAME = "net.i2p.util.messages";
 
     private static String _t(String key) {
         return Translate.getString(key, I2PAppContext.getGlobalContext(), BUNDLE_NAME);
@@ -1707,7 +1739,20 @@ public class DataHelper {
 
     /** */
     public static final int MAX_UNCOMPRESSED = 40*1024;
+    /**
+     *  Appx. 30% slower, 2.5% smaller than MEDIUM_COMPRESSION
+     */
     public static final int MAX_COMPRESSION = Deflater.BEST_COMPRESSION;
+    /**
+     *  Appx. 15% slower, 1.5% smaller than MEDIUM_COMPRESSION
+     *  @since 0.9.47
+     */
+    public static final int HIGH_COMPRESSION = 5;
+    /**
+     *  New default as of 0.9.47
+     *  @since 0.9.47
+     */
+    public static final int MEDIUM_COMPRESSION = 3;
     public static final int NO_COMPRESSION = Deflater.NO_COMPRESSION;
 
     /**
@@ -1719,6 +1764,10 @@ public class DataHelper {
      *  for a zero-length input. As of 0.9.29, output is valid for
      *  a zero-length input also.
      *
+     *  As of 0.9.47, this uses a level of MEDIUM_COMPRESSION,
+     *  which is a good space/speed tradeoff.
+     *  Prior to that, it used MAX_COMPRESSION.
+     *
      *  @throws IllegalArgumentException if input size is over 40KB
      *  @throws IllegalStateException on compression failure, as of 0.9.29
      *  @return null if orig is null
@@ -1736,12 +1785,16 @@ public class DataHelper {
      *  for a zero-length input. As of 0.9.29, output is valid for
      *  a zero-length input also.
      *
+     *  As of 0.9.47, this uses a level of MEDIUM_COMPRESSION,
+     *  which is a good space/speed tradeoff.
+     *  Prior to that, it used MAX_COMPRESSION.
+     *
      *  @throws IllegalArgumentException if size is over 40KB
      *  @throws IllegalStateException on compression failure, as of 0.9.29
      *  @return null if orig is null
      */
     public static byte[] compress(byte orig[], int offset, int size) {
-        return compress(orig, offset, size, MAX_COMPRESSION);
+        return compress(orig, offset, size, MEDIUM_COMPRESSION);
     }
 
     /**
@@ -1760,6 +1813,8 @@ public class DataHelper {
      */
     public static byte[] compress(byte orig[], int offset, int size, int level) {
         if (orig == null) return orig;
+        if (level == NO_COMPRESSION && size <= 32767)
+            return zeroCompress(orig, offset, size);
         if (size > MAX_UNCOMPRESSED) 
             throw new IllegalArgumentException("tell jrandom size=" + size);
         ReusableGZIPOutputStream out = ReusableGZIPOutputStream.acquire();
@@ -1794,7 +1849,41 @@ public class DataHelper {
         }
         
     }
-    
+
+    /**
+     *  Fast NO_COMPRESSION.
+     *  135x faster for 23 bytes, 15x faster for 1752 bytes.
+     *
+     *  Ref: RFC 1951, RFC 1952, ResettableGzipOutputStream
+     *
+     *  @param len 32767 max
+     *  @return 23 bytes longer
+     *  @since 0.9.46
+     */
+    private static byte[] zeroCompress(byte[] in, int off, int len) {
+        if (len > 32767)
+            throw new IllegalArgumentException();
+        byte[] rv = new byte[len + 23];
+        rv[0] = 0x1F;
+        rv[1] = (byte) 0x8B;
+        rv[2] = 0x08;
+        rv[8] = 0x02;         // XFL max compression slowest algorithm
+        rv[9] = (byte) 0xFF;  // unknown OS
+        rv[10] = 0x01;        // BFINAL, BTYPE = 0 (no compression)
+        rv[11] = (byte) (len & 0xff);
+        rv[12] = (byte) ((len >> 8) & 0xff);
+        rv[13] = (byte) ~ rv[11];
+        rv[14] = (byte) ~ rv[12];
+        System.arraycopy(in, off, rv, 15, len);
+        CRC32 crc = new CRC32();
+        crc.update(in, off, len);
+        long val = crc.getValue();
+        toLongLE(rv, rv.length - 8, 4, val);
+        rv[rv.length - 4] = rv[11];
+        rv[rv.length - 3] = rv[12];
+        return rv;
+    }
+
     /**
      *  Decompress the GZIP compressed data (returning null on error).
      *  @throws IOException if uncompressed is over 40 KB,
@@ -1813,6 +1902,10 @@ public class DataHelper {
      */
     public static byte[] decompress(byte orig[], int offset, int length) throws IOException {
         if (orig == null) return orig;
+        if (length < 23)
+            throw new IOException("length");
+        if (length < 65559 && orig[offset + 10] == 0x01)
+            return zeroDecompress(orig, offset, length);
         if (offset + length > orig.length)
             throw new IOException("Bad params arrlen " + orig.length + " off " + offset + " len " + length);
         
@@ -1844,6 +1937,53 @@ public class DataHelper {
         }
     }
 
+    /**
+     *  Fast NO_COMPRESSION.
+     *  ~30x faster for typ. lengths, ~20x faster for 4 KB or more.
+     *
+     *  Ref: RFC 1951, RFC 1952, ResettableGzipOutputStream
+     *
+     *  @param in in[off + 10] MUST BE VALIDATED == 1 before calling this.
+     *  @param len 65558 max
+     *  @return 23 bytes shorter
+     *  @throws IOException on all errors
+     *  @since 0.9.47
+     */
+    private static byte[] zeroDecompress(byte[] in, int off, int len) throws IOException {
+        if (len > 65535 + 23)
+            throw new IOException("length");
+        try {
+            final int olen = len - 23;
+            if (in[off++] != 0x1F ||
+                in[off++] != (byte) 0x8B ||
+                in[off] != 0x08)
+                throw new IOException("header");
+            off += 8;
+            if (in[off++] != 0x01 ||
+                fromLongLE(in, off, 2) != olen)
+                throw new IOException("header");
+            off += 2;
+            if (in[off] != (byte) ~ in[off - 2] ||
+                in[off + 1] != (byte) ~ in[off - 1])
+                throw new IOException("header");
+            off += 2;
+            final int trailer = off + olen;
+            if (fromLongLE(in, trailer + 4, 4) != olen)
+                throw new IOException("trailer");
+            CRC32 crc = new CRC32();
+            crc.update(in, off, olen);
+            long val = crc.getValue();
+            if (val != fromLongLE(in, trailer, 4))
+                throw new IOException("CRC");
+            final byte[] rv = new byte[olen];
+            System.arraycopy(in, off, rv, 0, olen);
+            return rv;
+        } catch (ArrayIndexOutOfBoundsException aioobe) {
+            throw new IOException(aioobe);
+        }
+    }
+
+
     /**
      *  Same as orig.getBytes("UTF-8") but throws an unchecked RuntimeException
      *  instead of an UnsupportedEncodingException if no UTF-8, for ease of use.
