@@ -21,7 +21,7 @@ import net.i2p.util.Log;
 import net.i2p.util.OrderedProperties;
 
 /**
- * PRELIMINARY - Subject to change - see proposal 123
+ * See proposal 123
  *
  * @since 0.9.38
  */
@@ -44,6 +44,9 @@ public class LeaseSet2 extends LeaseSet {
     // If this leaseset was formerly blinded, the blinded hash, so we can find it again
     private Hash _blindedHash;
 
+    // true for testing
+    private static final boolean IGNORE_SERVER_KEY_PREFERENCE = false;
+
     private static final int FLAG_OFFLINE_KEYS = 0x01;
     private static final int FLAG_UNPUBLISHED = 0x02;
     /**
@@ -85,7 +88,12 @@ public class LeaseSet2 extends LeaseSet {
         return (_flags & FLAG_UNPUBLISHED) != 0;
     }
 
+    /**
+     *  @throws IllegalStateException if already signed
+     */
     public void setUnpublished() {
+        if (_signature != null && (_flags & FLAG_UNPUBLISHED) == 0)
+            throw new IllegalStateException();
         _flags |= FLAG_UNPUBLISHED;
     }
 
@@ -99,9 +107,12 @@ public class LeaseSet2 extends LeaseSet {
 
     /**
      *  Set if the unencrypted LS, when published, will be blinded/encrypted
+     *  @throws IllegalStateException if already signed
      *  @since 0.9.42
      */
     public void setBlindedWhenPublished() {
+        if (_signature != null && (_flags & FLAG_BLINDED) == 0)
+            throw new IllegalStateException();
         _flags |= FLAG_BLINDED;
     }
     
@@ -150,9 +161,25 @@ public class LeaseSet2 extends LeaseSet {
      */
     @Override
     public PublicKey getEncryptionKey(Set<EncType> supported) {
-        for (PublicKey pk : getEncryptionKeys()) {
-            if (supported.contains(pk.getType()))
-                return pk;
+        List<PublicKey> keys = getEncryptionKeys();
+        if (keys == null)
+            return null;
+        if (!IGNORE_SERVER_KEY_PREFERENCE || supported.size() <= 1 || keys.size() <= 1) {
+            // Honor order in LS
+            for (PublicKey pk : keys) {
+                if (supported.contains(pk.getType()))
+                    return pk;
+            }
+        } else {
+            // Our preference, newest enc type first
+            List<EncType> types = new ArrayList<EncType>(supported);
+            Collections.sort(types, Collections.reverseOrder());
+            for (EncType type : types) {
+                for (PublicKey pk : keys) {
+                    if (type == pk.getType())
+                        return pk;
+                }
+            }
         }
         return null;
     }
@@ -232,8 +259,11 @@ public class LeaseSet2 extends LeaseSet {
      *  @param transientSPK the key that will sign the leaseset
      *  @param offlineSig the signature by the spk in the destination
      *  @return success, false if verify failed or expired
+     *  @throws IllegalStateException if already signed
      */
     public boolean setOfflineSignature(long expires, SigningPublicKey transientSPK, Signature offlineSig) {
+        if (_signature != null)
+            throw new IllegalStateException();
         _flags |= FLAG_OFFLINE_KEYS;
         _transientExpires = expires;
         _transientSigningPublicKey = transientSPK;
@@ -672,6 +702,7 @@ public class LeaseSet2 extends LeaseSet {
             }
         }
         buf.append("\n\tUnpublished? ").append(isUnpublished());
+        buf.append("\n\tBlinded? ").append(isBlindedWhenPublished());
         buf.append("\n\tSignature: ").append(_signature);
         buf.append("\n\tPublished: ").append(new java.util.Date(_published));
         buf.append("\n\tExpires: ").append(new java.util.Date(_expires));
