@@ -32,7 +32,7 @@ public class LeaseSetKeys {
      * @since 0.9.44
      */
     public static final Set<EncType> SET_ELG = Collections.unmodifiableSet(EnumSet.of(EncType.ELGAMAL_2048));
-    private static final Set<EncType> SET_EC = Collections.unmodifiableSet(EnumSet.of(EncType.ECIES_X25519));
+    public static final Set<EncType> SET_EC = Collections.unmodifiableSet(EnumSet.of(EncType.ECIES_X25519));
     private static final Set<EncType> SET_BOTH = Collections.unmodifiableSet(EnumSet.of(EncType.ELGAMAL_2048, EncType.ECIES_X25519));
     private static final Set<EncType> SET_NONE = Collections.unmodifiableSet(EnumSet.noneOf(EncType.class));
 
@@ -62,10 +62,12 @@ public class LeaseSetKeys {
      *
      *  @param dest unused
      *  @param revocationKey unused, may be null
-     *  @param decryptionKeys non-null
+     *  @param decryptionKeys non-null, non-empty
      *  @since 0.9.44
      */
     public LeaseSetKeys(Destination dest, SigningPrivateKey revocationKey, List<PrivateKey> decryptionKeys) {
+        if (decryptionKeys.isEmpty())
+            throw new IllegalArgumentException("no keys");
         _revocationKey = revocationKey;
         PrivateKey elg = null;
         PrivateKey ec = null;
