@@ -706,4 +706,24 @@ public class RouterContext extends I2PAppContext {
     public ECIESAEADEngine eciesEngine() {
         return _eciesEngine;
     }
+
+    /** 
+     *  How long this router was down before it started, or 0 if unknown.
+     *
+     *  This may be used for a determination of whether to regenerate keys, for example.
+     *  We use the timestamp of the previous ping file left behind on crash,
+     *  as set by isOnlyRouterRunning(), if present.
+     *  Otherwise, the last STOPPED entry in the event log.
+     *
+     *  May take a while to run the first time, if it has to go through the event log.
+     *  Once called, the result is cached.
+     *
+     *  @return downtime in ms or 0 if unknown
+     *  @since 0.0.47
+     */
+    public long getEstimatedDowntime() {
+        if (_router == null)
+            return 0L;
+        return _router.getEstimatedDowntime();
+    }
 }
