@@ -577,7 +577,9 @@ class ClientConnectionRunner {
             int thresh = TransientSessionKeyManager.LOW_THRESHOLD;
             boolean hasElg = false;
             boolean hasEC = false;
-            if (opts != null) {
+            // router may be null in unit tests, avoid NPEs in ratchet
+            // we won't actually be using any SKM anyway
+            if (opts != null && _context.router() != null) {
                 String ptags = opts.getProperty(PROP_TAGS);
                 if (ptags != null) {
                     try { tags = Integer.parseInt(ptags); } catch (NumberFormatException nfe) {}
@@ -602,16 +604,16 @@ class ClientConnectionRunner {
                 hasElg = true;
             }
             if (hasElg) {
+                TransientSessionKeyManager tskm = new TransientSessionKeyManager(_context, tags, thresh);
                 if (hasEC) {
-                    TransientSessionKeyManager tskm = new TransientSessionKeyManager(_context, tags, thresh);
-                    RatchetSKM rskm = new RatchetSKM(_context);
+                    RatchetSKM rskm = new RatchetSKM(_context, dest);
                     _sessionKeyManager = new MuxedSKM(tskm, rskm);
                 } else {
-                    _sessionKeyManager = new TransientSessionKeyManager(_context, tags, thresh);
+                    _sessionKeyManager = tskm;
                 }
             } else {
                 if (hasEC) {
-                    _sessionKeyManager = new RatchetSKM(_context);
+                    _sessionKeyManager = new RatchetSKM(_context, dest);
                 } else {
                     _log.error("No supported encryption types in i2cp.leaseSetEncType for " + dest.toBase32());
                     return SessionStatusMessage.STATUS_INVALID;
@@ -631,7 +633,7 @@ class ClientConnectionRunner {
      *
      *  @param dest the client
      *  @param id the router's ID for this message
-     *  @param messageNonce the client's ID for this message
+     *  @param messageNonce the client's ID for this message, greater than zero
      *  @param status see I2CP MessageStatusMessage for success/failure codes
      */
     void updateMessageDeliveryStatus(Destination dest, MessageId id, long messageNonce, int status) {
