@@ -19,11 +19,13 @@ import net.i2p.crypto.SessionKeyManager;
 import net.i2p.data.Certificate;
 import net.i2p.data.DataFormatException;
 import net.i2p.data.DataHelper;
+import net.i2p.data.Destination;
 import net.i2p.data.Hash;
 import net.i2p.data.PrivateKey;
 import net.i2p.data.PublicKey;
 import net.i2p.data.SessionKey;
 import net.i2p.data.SessionTag;
+import net.i2p.data.i2np.DeliveryInstructions;
 import net.i2p.data.i2np.GarlicClove;
 import net.i2p.data.i2np.GarlicMessage;
 import net.i2p.data.i2np.I2NPMessage;
@@ -31,6 +33,8 @@ import net.i2p.router.LeaseSetKeys;
 import net.i2p.router.RouterContext;
 import net.i2p.router.crypto.ratchet.MuxedSKM;
 import net.i2p.router.crypto.ratchet.RatchetSKM;
+import net.i2p.router.crypto.ratchet.RatchetSessionTag;
+import net.i2p.router.crypto.ratchet.ReplyCallback;
 import net.i2p.util.Log;
 
 /**
@@ -194,7 +198,7 @@ public class GarlicMessageBuilder {
     /**
      *  ELGAMAL_2048 only.
      *  Used by TestJob, and directly above,
-     *  and by MessageWrapper for encrypting DatabaseLookupMessages
+     *  and by MessageWrapper for encrypting DatabaseLookupMessages and DSM/DSRM replies.
      *
      * @param ctx scope
      * @param config how/what to wrap
@@ -235,28 +239,53 @@ public class GarlicMessageBuilder {
                 log.debug("Building a message expiring in " + timeFromNow + "ms: " + config, new Exception("created by"));
             return null;
         }
-        
-        if (log.shouldLog(Log.DEBUG))
-            log.debug("CloveSet (" + config.getCloveCount() + " cloves) for message " + msg.getUniqueId() + " is " + cloveSet.length
-                     + " bytes and encrypted message data is " + encData.length + " bytes");
-        
+        if (log.shouldDebug())
+            log.debug("Built ElG CloveSet (" + config.getCloveCount() + " cloves " + cloveSet.length + " bytes) in " + msg);
+        return msg;
+    }
+    
+    /**
+     *  Ratchet only.
+     *  Used by TestJob,
+     *  and by MessageWrapper for encrypting DatabaseLookupMessages and DSM/DSRM replies.
+     *
+     * @param ctx scope
+     * @param config how/what to wrap
+     * @param encryptKey sessionKey used to encrypt the current message, non-null
+     * @param encryptTag sessionTag used to encrypt the current message, non-null
+     * @since 0.9.46
+     */
+    public static GarlicMessage buildMessage(RouterContext ctx, GarlicConfig config,
+                                             SessionKey encryptKey, RatchetSessionTag encryptTag) {
+        GarlicMessage msg = new GarlicMessage(ctx);
+        CloveSet cloveSet = buildECIESCloveSet(ctx, config);
+        byte encData[] = ctx.eciesEngine().encrypt(cloveSet, encryptKey, encryptTag);
+        if (encData == null)
+            return null;
+        msg.setData(encData);
+        msg.setMessageExpiration(config.getExpiration());
+        Log log = ctx.logManager().getLog(GarlicMessageBuilder.class);
+        if (log.shouldDebug())
+            log.debug("Built ECIES CloveSet (" + config.getCloveCount() + " cloves) in " + msg);
         return msg;
     }
     
     /**
      * ECIES_X25519 only.
-     * Called by GarlicMessageBuilder only.
+     * Called by OCMJH only.
      *
      * @param ctx scope
      * @param config how/what to wrap
      * @param target public key of the location being garlic routed to (may be null if we 
      *               know the encryptKey and encryptTag)
+     * @param callback may be null
      * @return null if expired or on other errors
      * @throws IllegalArgumentException on error
      * @since 0.9.44
      */
     static GarlicMessage buildECIESMessage(RouterContext ctx, GarlicConfig config,
-                                           PublicKey target, Hash from, SessionKeyManager skm) {
+                                           PublicKey target, Hash from, Destination to, SessionKeyManager skm,
+                                           ReplyCallback callback) {
         PublicKey key = config.getRecipientPublicKey();
         if (key.getType() != EncType.ECIES_X25519)
             throw new IllegalArgumentException();
@@ -286,7 +315,7 @@ public class GarlicMessageBuilder {
                 log.warn("No SKM for " + from.toBase32());
             return null;
         }
-        byte encData[] = ctx.eciesEngine().encrypt(cloveSet, target, priv, rskm);
+        byte encData[] = ctx.eciesEngine().encrypt(cloveSet, target, to, priv, rskm, callback);
         if (encData == null) {
             if (log.shouldWarn())
                 log.warn("Encrypt fail for " + from.toBase32());
@@ -301,8 +330,7 @@ public class GarlicMessageBuilder {
             return null;
         }
         if (log.shouldDebug())
-            log.debug("CloveSet (" + config.getCloveCount() + " cloves) for message " + msg.getUniqueId()
-                     + " encrypted message data is " + encData.length + " bytes");
+            log.debug("Built ECIES CloveSet (" + config.getCloveCount() + " cloves) in " + msg);
         return msg;
     }
     
@@ -438,6 +466,11 @@ public class GarlicMessageBuilder {
         return rv;
     }
     
+    /**
+     * Build a single clove
+     *
+     * @since 0.9.44
+     */
     private static GarlicClove buildECIESClove(RouterContext ctx, PayloadGarlicConfig config) {
         GarlicClove clove = new GarlicClove(ctx);
         clove.setData(config.getPayload());
