@@ -35,7 +35,7 @@ import net.i2p.util.Log;
 class FloodfillVerifyStoreJob extends JobImpl {
     private final Log _log;
     private final Hash _key, _client;
-    private Hash _target;
+    private volatile Hash _target;
     private final Hash _sentTo;
     private final FloodfillNetworkDatabaseFacade _facade;
     private long _expiration;
@@ -146,16 +146,23 @@ class FloodfillVerifyStoreJob extends JobImpl {
             _facade.verifyFinished(_key);
             return;
         }
+        boolean supportsElGamal = true;
+        boolean supportsRatchet = false;
         if (DatabaseLookupMessage.supportsEncryptedReplies(peer)) {
             // register the session with the right SKM
             MessageWrapper.OneTimeSession sess;
             if (isInboundExploratory) {
-                sess = MessageWrapper.generateSession(getContext());
+                sess = MessageWrapper.generateSession(getContext(), VERIFY_TIMEOUT);
             } else {
                 LeaseSetKeys lsk = getContext().keyManager().getKeys(_client);
-                if (lsk == null || lsk.isSupported(EncType.ELGAMAL_2048)) {
+                supportsRatchet = lsk != null &&
+                                  lsk.isSupported(EncType.ECIES_X25519) &&
+                                  DatabaseLookupMessage.supportsRatchetReplies(peer);
+                supportsElGamal = lsk != null &&
+                                  lsk.isSupported(EncType.ELGAMAL_2048);
+                if (supportsElGamal || supportsRatchet) {
                     // garlic encrypt
-                    sess = MessageWrapper.generateSession(getContext(), _client);
+                    sess = MessageWrapper.generateSession(getContext(), _client, VERIFY_TIMEOUT, !supportsRatchet);
                     if (sess == null) {
                          if (_log.shouldLog(Log.WARN))
                              _log.warn("No SKM to reply to");
@@ -163,7 +170,7 @@ class FloodfillVerifyStoreJob extends JobImpl {
                         return;
                     }
                 } else {
-                    // We don't yet have any way to request/get a ECIES-tagged reply,
+                    // We don't have a compatible way to get a reply,
                     // skip it for now.
                      if (_log.shouldWarn())
                          _log.warn("Skipping store verify for ECIES client " + _client.toBase32());
@@ -171,23 +178,41 @@ class FloodfillVerifyStoreJob extends JobImpl {
                     return;
                 }
             }
-            if (_log.shouldLog(Log.INFO))
-                _log.info(getJobId() + ": Requesting encrypted reply from " + _target + ' ' + sess.key + ' ' + sess.tag);
-            lookup.setReplySession(sess.key, sess.tag);
+            if (sess.tag != null) {
+                if (_log.shouldInfo())
+                    _log.info(getJobId() + ": Requesting AES reply from " + peer + ' ' + sess.key + ' ' + sess.tag);
+                lookup.setReplySession(sess.key, sess.tag);
+            } else {
+                if (_log.shouldInfo())
+                    _log.info(getJobId() + ": Requesting AEAD reply from " + peer + ' ' + sess.key + ' ' + sess.rtag);
+                lookup.setReplySession(sess.key, sess.rtag);
+            }
         }
         Hash fromKey;
-        if (_isRouterInfo)
-            fromKey = null;
-        else
-            fromKey = _client;
-        _wrappedMessage = MessageWrapper.wrap(getContext(), lookup, fromKey, peer);
-        if (_wrappedMessage == null) {
-             if (_log.shouldLog(Log.WARN))
-                _log.warn("Fail Garlic encrypting");
-            _facade.verifyFinished(_key);
-            return;
+        I2NPMessage sent;
+        if (supportsElGamal) {
+            if (_isRouterInfo)
+                fromKey = null;
+            else
+                fromKey = _client;
+            _wrappedMessage = MessageWrapper.wrap(getContext(), lookup, fromKey, peer);
+            if (_wrappedMessage == null) {
+                 if (_log.shouldLog(Log.WARN))
+                    _log.warn("Fail Garlic encrypting");
+                _facade.verifyFinished(_key);
+                return;
+            }
+            sent = _wrappedMessage.getMessage();
+        } else {
+            // force full ElG for ECIES fromkey
+            sent = MessageWrapper.wrap(getContext(), lookup, peer);
+            if (sent == null) {
+                 if (_log.shouldLog(Log.WARN))
+                    _log.warn("Fail Garlic encrypting");
+                _facade.verifyFinished(_key);
+                return;
+            }
         }
-        I2NPMessage sent = _wrappedMessage.getMessage();
 
         if (_log.shouldLog(Log.INFO))
             _log.info(getJobId() + ": Starting verify (stored " + _key + " to " + _sentTo + "), asking " + _target);
@@ -304,6 +329,15 @@ class FloodfillVerifyStoreJob extends JobImpl {
                 // Verify it's as recent as the one we sent
                 DatabaseStoreMessage dsm = (DatabaseStoreMessage)_message;
                 DatabaseEntry entry = dsm.getEntry();
+                if (!entry.verifySignature()) {
+                    if (_log.shouldWarn())
+                        _log.warn(getJobId() + ": Sent bad data for verify: " + _target);
+                    pm.dbLookupFailed(_target);
+                    ctx.banlist().banlistRouterForever(_target, "Sent bad netdb data");
+                    ctx.statManager().addRateData("netDb.floodfillVerifyFail", delay);
+                    resend();
+                    return;
+                }
                 long newDate;
                 boolean success;
                 if (_isLS2 &&
@@ -351,7 +385,7 @@ class FloodfillVerifyStoreJob extends JobImpl {
             // but we have to use one of them to affect the FloodfillPeerSelector ordering.
             // If we don't do this we get stuck using the same verify peer every time even
             // though it is the real problem.
-            if (_target != null && !_target.equals(_sentTo))
+            if (!_target.equals(_sentTo))
                 pm.dbLookupFailed(_target);
             ctx.statManager().addRateData("netDb.floodfillVerifyFail", delay);
             resend();
@@ -391,8 +425,7 @@ class FloodfillVerifyStoreJob extends JobImpl {
             Set<Hash> toSkip = new HashSet<Hash>(2);
             if (_sentTo != null)
                 toSkip.add(_sentTo);
-            if (_target != null)
-                toSkip.add(_target);
+            toSkip.add(_target);
             if (_log.shouldWarn())
                 _log.warn(getJobId() + ": Verify failed, starting new store for: " + _key);
             _facade.sendStore(_key, ds, null, null, FloodfillNetworkDatabaseFacade.PUBLISH_TIMEOUT, toSkip);
