@@ -23,6 +23,9 @@ import net.i2p.util.Log;
  * at a time.
  * If the explore pool is empty, just search for a random key.
  *
+ * For hidden mode routers, this is the primary mechanism for staying integrated.
+ * The goal is to keep known router count above LOW_ROUTERS and
+ * the known floodfill count above LOW_FFS.
  */
 class StartExplorersJob extends JobImpl {
     private final Log _log;
@@ -31,17 +34,23 @@ class StartExplorersJob extends JobImpl {
     /** don't explore more than 1 bucket at a time */
     private static final int MAX_PER_RUN = 1;
     /** dont explore the network more often than this */
-    private static final int MIN_RERUN_DELAY_MS = 99*1000;
+    private static final int MIN_RERUN_DELAY_MS = 55*1000;
     /** explore the network at least this often */
     private static final int MAX_RERUN_DELAY_MS = 15*60*1000;
     /** aggressively explore during this time - same as KNDF expiration grace period */
     private static final int STARTUP_TIME = 60*60*1000;
-    /** super-aggressively explore if we have less than this many routers */
-    private static final int LOW_ROUTERS = 125;
+    /** super-aggressively explore if we have less than this many routers.
+        The goal here is to avoid reseeding.
+     */
+    /** very aggressively explore if we have less than this many routers */
+    private static final int MIN_ROUTERS = 3 * KademliaNetworkDatabaseFacade.MIN_RESEED;
     /** aggressively explore if we have less than this many routers */
-    private static final int MIN_ROUTERS = 250;
+    private static final int LOW_ROUTERS = 2 * MIN_ROUTERS;
     /** explore slowly if we have more than this many routers */
-    private static final int MAX_ROUTERS = 800;
+    private static final int MAX_ROUTERS = 2 * LOW_ROUTERS;
+    private static final int MIN_FFS = 50;
+    static final int LOW_FFS = 2 * MIN_FFS;
+
     private static final long MAX_LAG = 100;
     private static final long MAX_MSG_DELAY = 1500;
     
@@ -60,21 +69,37 @@ class StartExplorersJob extends JobImpl {
                // message delay limit also?
                getContext().router().gracefulShutdownInProgress())) {
             int num = MAX_PER_RUN;
-            if (_facade.getDataStore().size() < LOW_ROUTERS)
-                num *= 3;
+            int count = _facade.getDataStore().size();
+            if (count < MIN_ROUTERS)
+                num *= 15;  // at less than 3x MIN_RESEED, explore extremely aggressively
+            else if (count < LOW_ROUTERS)
+                num *= 10;  // 3x was not sufficient to keep hidden routers from losing peers
             if (getContext().router().getUptime() < STARTUP_TIME)
-                num *= 3;
+                num *= 2;
             Set<Hash> toExplore = selectKeysToExplore(num);
             if (_log.shouldLog(Log.DEBUG))
-                _log.debug("Keys to explore during this run: " + toExplore);
+                _log.debug("Keys to explore during this run: " + toExplore + ", wanted " + num + ", got " + toExplore.size());
             _facade.removeFromExploreKeys(toExplore);
             long delay = 0;
+
+            // If we're below about 30 ffs, standard exploration stops working well.
+            // A non-exploratory "exploration" finds us floodfills quickly.
+            // This is vital when in hidden mode, where this is our primary method
+            // of maintaining sufficient peers and avoiding repeated reseeding.
+            int ffs = getContext().peerManager().countPeersByCapability(FloodfillNetworkDatabaseFacade.CAPABILITY_FLOODFILL);
+            boolean needffs = ffs < MIN_FFS;
+            boolean lowffs = ffs < LOW_FFS;
             for (Hash key : toExplore) {
-                ExploreJob j = new ExploreJob(getContext(), _facade, key);
+                // Last param false means get floodfills (non-explore)
+                // This is very effective so we don't need to do it often
+                boolean realexpl = !((needffs && getContext().random().nextInt(2) == 0) ||
+                                    (lowffs && getContext().random().nextInt(4) == 0));
+                ExploreJob j = new ExploreJob(getContext(), _facade, key, realexpl);
                 if (delay > 0)
                     j.getTiming().setStartAfter(getContext().clock().now() + delay);
                 getContext().jobQueue().addJob(j);
-                delay += 200;
+                // spread them out
+                delay += 1000 + getContext().random().nextInt(512);
             }
         }
         long delay = getNextRunDelay();
@@ -141,8 +166,8 @@ class StartExplorersJob extends JobImpl {
             _log.debug("Keys waiting for exploration: " + queued.size());
         Set<Hash> rv = new HashSet<Hash>(num);
         for (Hash key : queued) {
-            if (rv.size() >= num) break;
             rv.add(key);
+            if (rv.size() >= num) break;
         }
         for (int i = rv.size(); i < num; i++) {
             byte hash[] = new byte[Hash.HASH_LENGTH];
