@@ -10,6 +10,7 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.InputStreamReader;
 import java.util.Arrays;
+import java.util.Date;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
@@ -22,6 +23,7 @@ import com.maxmind.geoip.LookupService;
 import com.maxmind.geoip2.DatabaseReader;
 
 import net.i2p.I2PAppContext;
+import net.i2p.app.ClientAppManager;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Hash;
 import net.i2p.data.router.RouterAddress;
@@ -29,6 +31,8 @@ import net.i2p.data.router.RouterInfo;
 import net.i2p.router.Router;
 import net.i2p.router.RouterContext;
 import net.i2p.router.transport.udp.UDPTransport;
+import net.i2p.update.UpdateManager;
+import net.i2p.update.UpdateType;
 import net.i2p.util.Addresses;
 import net.i2p.util.ConcurrentHashSet;
 import net.i2p.util.Log;
@@ -69,7 +73,7 @@ public class GeoIP {
     public static final String PROP_GEOIP_DIR = "geoip.dir";
     public static final String GEOIP_DIR_DEFAULT = "geoip";
     static final String GEOIP_FILE_DEFAULT = "geoip.txt";
-    static final String GEOIP2_FILE_DEFAULT = "GeoLite2-Country.mmdb";
+    public static final String GEOIP2_FILE_DEFAULT = "GeoLite2-Country.mmdb";
     static final String COUNTRY_FILE_DEFAULT = "countries.txt";
     public static final String PROP_IP_COUNTRY = "i2np.lastCountry";
     public static final String PROP_DEBIAN_GEOIP = "geoip.dat";
@@ -80,6 +84,8 @@ public class GeoIP {
     private static final boolean ENABLE_DEBIAN = !DISABLE_DEBIAN && !(SystemVersion.isWindows() || SystemVersion.isAndroid());
     /** maxmind API */
     private static final String UNKNOWN_COUNTRY_CODE = "--";
+    /** db-ip.com https://db-ip.com/faq.php */
+    private static final String UNKNOWN_COUNTRY_CODE2 = "ZZ";
 
     /**
      *  @param context RouterContext in production, I2PAppContext for testing only
@@ -156,6 +162,8 @@ public class GeoIP {
         public void run() {
             if (_lock.getAndSet(true))
                 return;
+            int toSearch = 0;
+            int found = 0;
             File geoip2 = getGeoIP2();
             DatabaseReader dbr = null;
             long start = _context.clock().now();
@@ -178,13 +186,21 @@ public class GeoIP {
                 Long[] search = _pendingSearch.toArray(new Long[_pendingSearch.size()]);
                 _pendingSearch.clear();
                 if (search.length > 0) {
+                    toSearch += search.length;
                     Arrays.sort(search);
                     File f = new File(_context.getProperty(PROP_DEBIAN_GEOIP, DEBIAN_GEOIP_FILE));
-                    if (ENABLE_DEBIAN && f.exists()) {
+                    // if we have both, prefer the most recent.
+                    // The Debian data can be pretty old.
+                    // For now, we use the file date, we don't open it up to get the metadata.
+                    if (ENABLE_DEBIAN && f.exists() &&
+                        (geoip2 == null || f.lastModified() > geoip2.lastModified())) {
                         // Maxmind v1 database
                         LookupService ls = null;
                         try {
                             ls = new LookupService(f, LookupService.GEOIP_STANDARD);
+                            Date date = ls.getDatabaseInfo().getDate();
+                            if (date != null)
+                                notifyVersion("GeoIPv4", date.getTime());
                             for (int i = 0; i < search.length; i++) {
                                 Long ipl = search[i];
                                 long ip = ipl.longValue();
@@ -196,6 +212,7 @@ public class GeoIP {
                                     if (cached == null)
                                         cached = lc;
                                     _IPToCountry.put(ipl, cached);
+                                    found++;
                                 } else {
                                     _notFound.add(ipl);
                                 }
@@ -216,12 +233,13 @@ public class GeoIP {
                                 String ipv4 = toV4(ipl);
                                 // returns upper case or null
                                 String uc = dbr.country(ipv4);
-                                if (uc != null) {
+                                if (uc != null && !uc.equals(UNKNOWN_COUNTRY_CODE2)) {
                                     String lc = uc.toLowerCase(Locale.US);
                                     String cached = _codeCache.get(lc);
                                     if (cached == null)
                                         cached = lc;
                                     _IPToCountry.put(ipl, cached);
+                                    found++;
                                 } else {
                                     _notFound.add(ipl);
                                 }
@@ -233,10 +251,12 @@ public class GeoIP {
                         // Tor-style database
                         String[] countries = readGeoIPFile(search);
                         for (int i = 0; i < countries.length; i++) {
-                            if (countries[i] != null)
+                            if (countries[i] != null) {
                                 _IPToCountry.put(search[i], countries[i]);
-                            else
+                                found++;
+                            } else {
                                 _notFound.add(search[i]);
+                            }
                         }
                     }
                 }
@@ -244,13 +264,18 @@ public class GeoIP {
                 search = _pendingIPv6Search.toArray(new Long[_pendingIPv6Search.size()]);
                 _pendingIPv6Search.clear();
                 if (search.length > 0) {
+                    toSearch += search.length;
                     Arrays.sort(search);
                     File f = new File(_context.getProperty(PROP_DEBIAN_GEOIPV6, DEBIAN_GEOIPV6_FILE));
-                    if (ENABLE_DEBIAN && f.exists()) {
+                    if (ENABLE_DEBIAN && f.exists() &&
+                        (geoip2 == null || f.lastModified() > geoip2.lastModified())) {
                         // Maxmind v1 database
                         LookupService ls = null;
                         try {
                             ls = new LookupService(f, LookupService.GEOIP_STANDARD);
+                            Date date = ls.getDatabaseInfo().getDate();
+                            if (date != null)
+                                notifyVersion("GeoIPv6", date.getTime());
                             for (int i = 0; i < search.length; i++) {
                                 Long ipl = search[i];
                                 long ip = ipl.longValue();
@@ -263,6 +288,7 @@ public class GeoIP {
                                     if (cached == null)
                                         cached = lc;
                                     _IPToCountry.put(ipl, cached);
+                                    found++;
                                 } else {
                                     _notFound.add(ipl);
                                 }
@@ -284,12 +310,13 @@ public class GeoIP {
                                 String ipv6 = toV6(ipl);
                                 // returns upper case or null
                                 String uc = dbr.country(ipv6);
-                                if (uc != null) {
+                                if (uc != null && !uc.equals(UNKNOWN_COUNTRY_CODE2)) {
                                     String lc = uc.toLowerCase(Locale.US);
                                     String cached = _codeCache.get(lc);
                                     if (cached == null)
                                         cached = lc;
                                     _IPToCountry.put(ipl, cached);
+                                    found++;
                                 } else {
                                     _notFound.add(ipl);
                                 }
@@ -301,10 +328,12 @@ public class GeoIP {
                         // I2P format IPv6 database
                         String[] countries = GeoIPv6.readGeoIPFile(_context, search, _codeCache);
                         for (int i = 0; i < countries.length; i++) {
-                            if (countries[i] != null)
+                            if (countries[i] != null) {
                                 _IPToCountry.put(search[i], countries[i]);
-                            else
+                                found++;
+                            } else {
                                 _notFound.add(search[i]);
+                            }
                         }
                     }
                 }
@@ -313,7 +342,8 @@ public class GeoIP {
                 _lock.set(false);
             }
             if (_log.shouldInfo())
-                _log.info("GeoIP processing finished, time: " + (_context.clock().now() - start));
+                _log.info("GeoIP processing finished, looked up: " + toSearch + " found: " + found +
+                          " time: " + (_context.clock().now() - start));
         }
     }
 
@@ -347,6 +377,8 @@ public class GeoIP {
         DatabaseReader rv = b.build();
         if (_log.shouldDebug())
             _log.debug("Opened GeoIP2 Database, Metadata: " + rv.getMetadata());
+        long time = rv.getMetadata().getBuildDate().getTime();
+        notifyVersion("GeoIP2", time);
         return rv;
     }
 
@@ -447,6 +479,7 @@ public class GeoIP {
             String buf = null;
             br = new BufferedReader(new InputStreamReader(
                     new FileInputStream(geoFile), "ISO-8859-1"));
+            notifyVersion("Torv4", geoFile.lastModified());
             while ((buf = br.readLine()) != null && idx < search.length) {
                 try {
                     if (buf.charAt(0) == '#') {
@@ -480,6 +513,31 @@ public class GeoIP {
         return rv;
     }
 
+    /**
+     *  Tell the update manager.
+     *
+     *  @since 0.9.45
+     */
+    private void notifyVersion(String subtype, long version) {
+        notifyVersion(_context, subtype, version);
+    }
+
+    /**
+     *  Tell the update manager.
+     *
+     *  @since 0.9.45
+     */
+    static void notifyVersion(I2PAppContext ctx, String subtype, long version) {
+        if (version <= 0)
+            return;
+        ClientAppManager cmgr = ctx.clientAppManager();
+        if (cmgr != null) {
+            UpdateManager umgr = (UpdateManager) cmgr.getRegisteredApp(UpdateManager.APP_NAME);
+            if (umgr != null)
+                umgr.notifyInstalled(UpdateType.GEOIP, subtype, Long.toString(version));
+        }
+    }
+
     /**
      *  Put our country code in the config, where others (such as Timestamper) can get it,
      *  and it will be there next time at startup.
@@ -660,9 +718,13 @@ public class GeoIP {
         return _codeToName.get(code);
     }
 
-/***
     public static void main(String args[]) {
+        if (args.length <= 0) {
+            System.out.println("Usage: GeoIP ip...");
+            System.exit(1);
+        }
         GeoIP g = new GeoIP(I2PAppContext.getGlobalContext());
+/***
         String tests[] = {"0.0.0.0", "0.0.0.1", "0.0.0.2", "0.0.0.255", "1.0.0.0",
                                         "94.3.3.3", "77.1.2.3", "127.0.0.0", "127.127.127.127", "128.0.0.0",
                                         "89.8.9.3", "72.5.6.8", "217.4.9.7", "175.107.027.107", "135.6.5.2",
@@ -672,12 +734,17 @@ public class GeoIP {
                           };
         for (int i = 0; i < tests.length; i++)
             g.add(tests[i]);
+***/
+        for (int i = 0; i < args.length; i++)
+            g.add(args[i]);
         long start = System.currentTimeMillis();
         g.blockingLookup();
         System.out.println("Lookup took " + (System.currentTimeMillis() - start));
+/***
         for (int i = 0; i < tests.length; i++)
             System.out.println(tests[i] + " : " + g.get(tests[i]));
-
-    }
 ***/
+        for (int i = 0; i < args.length; i++)
+            System.out.println(args[i] + " : " + g.get(args[i]));
+    }
 }
