@@ -88,6 +88,7 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
     private static final Set<State> STATES_NTCP2 =
         EnumSet.of(State.IB_NTCP2_INIT, State.IB_NTCP2_GOT_X, State.IB_NTCP2_GOT_PADDING,
                    State.IB_NTCP2_SENT_Y, State.IB_NTCP2_GOT_RI, State.IB_NTCP2_READ_RANDOM);
+    private static final Set<State> STATES_MSG3 = EnumSet.of(State.IB_SENT_Y, State.IB_GOT_RI_SIZE, State.IB_GOT_RI);
 
     
     public InboundEstablishState(RouterContext ctx, NTCPTransport transport, NTCPConnection con) {
@@ -270,9 +271,7 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
         }
 
         // ok, we are onto the encrypted area, i.e. Message #3
-        while ((_state == State.IB_SENT_Y ||
-                _state == State.IB_GOT_RI_SIZE ||
-                _state == State.IB_GOT_RI) && src.hasRemaining()) {
+        while (STATES_MSG3.contains(_state) && src.hasRemaining()) {
 
                 // Collect a 16-byte block
                 if (_received < AES_SIZE && src.hasRemaining()) {
@@ -370,7 +369,7 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
         }
 
         // check for remaining data
-        if ((_state == State.VERIFIED || _state == State.CORRUPT) && src.hasRemaining()) {
+        if (STATES_DONE.contains(_state) && src.hasRemaining()) {
             if (_log.shouldWarn())
                 _log.warn("Received unexpected " + src.remaining() + " on " + this, new Exception());
         }
@@ -592,7 +591,7 @@ class InboundEstablishState extends EstablishBase implements NTCP2Payload.Payloa
         if (!rv) {
             Hash aliceHash = alice.getHash();
             if (_log.shouldLog(Log.WARN))
-                _log.warn("Dropping inbound connection from wrong network: " + aliceID + ' ' + aliceHash);
+                _log.warn("Not in our network: " + alice, new Exception());
             // So next time we will not accept the con from this IP,
             // rather than doing the whole handshake
             InetAddress addr = _con.getChannel().socket().getInetAddress();
