@@ -1,12 +1,9 @@
 package net.i2p.router.tunnel.pool;
 
-import java.util.Set;
+import java.util.concurrent.atomic.AtomicInteger;
 
 import net.i2p.crypto.SessionKeyManager;
-import net.i2p.data.Certificate;
-import net.i2p.data.SessionKey;
 import net.i2p.data.SessionTag;
-import net.i2p.data.i2np.DeliveryInstructions;
 import net.i2p.data.i2np.DeliveryStatusMessage;
 import net.i2p.data.i2np.GarlicMessage;
 import net.i2p.data.i2np.I2NPMessage;
@@ -16,9 +13,7 @@ import net.i2p.router.OutNetMessage;
 import net.i2p.router.ReplyJob;
 import net.i2p.router.RouterContext;
 import net.i2p.router.TunnelInfo;
-import net.i2p.router.message.GarlicMessageBuilder;
-import net.i2p.router.message.PayloadGarlicConfig;
-import net.i2p.router.util.RemovableSingletonSet;
+import net.i2p.router.networkdb.kademlia.MessageWrapper;
 import net.i2p.stat.Rate;
 import net.i2p.stat.RateStat;
 import net.i2p.util.Log;
@@ -26,6 +21,9 @@ import net.i2p.util.Log;
 /**
  *  Repeatedly test a single tunnel for its entire lifetime,
  *  or until the pool is shut down or removed from the client manager.
+ *
+ *  Tunnel testing is disabled by default now, except for hidden mode,
+ *  see TunnelPoolManager.buildComplete()
  */
 class TestJob extends JobImpl {
     private final Log _log;
@@ -37,6 +35,8 @@ class TestJob extends JobImpl {
     private PooledTunnelCreatorConfig _otherTunnel;
     /** save this so we can tell the SKM to kill it if the test fails */
     private SessionTag _encryptTag;
+    private static final AtomicInteger __id = new AtomicInteger();
+    private int _id;
     
     /** base to randomize the test delay on */
     private static final int TEST_DELAY = 40*1000;
@@ -60,15 +60,16 @@ class TestJob extends JobImpl {
     public void runJob() {
         if (_pool == null || !_pool.isAlive())
             return;
-        long lag = getContext().jobQueue().getMaxLag();
+        final RouterContext ctx = getContext();
+        long lag = ctx.jobQueue().getMaxLag();
         if (lag > 3000) {
             if (_log.shouldLog(Log.WARN))
                 _log.warn("Deferring test of " + _cfg + " due to job lag = " + lag);
-            getContext().statManager().addRateData("tunnel.testAborted", _cfg.getLength(), 0);
+            ctx.statManager().addRateData("tunnel.testAborted", _cfg.getLength());
             scheduleRetest();
             return;
         }
-        if (getContext().router().gracefulShutdownInProgress())
+        if (ctx.router().gracefulShutdownInProgress())
             return;   // don't reschedule
         _found = false;
         // note: testing with exploratory tunnels always, even if the tested tunnel
@@ -80,11 +81,11 @@ class TestJob extends JobImpl {
         if (_cfg.isInbound()) {
             _replyTunnel = _cfg;
             // TODO if testing is re-enabled, pick closest to far end
-            _outTunnel = getContext().tunnelManager().selectOutboundTunnel();
+            _outTunnel = ctx.tunnelManager().selectOutboundTunnel();
             _otherTunnel = (PooledTunnelCreatorConfig) _outTunnel;
         } else {
             // TODO if testing is re-enabled, pick closest to far end
-            _replyTunnel = getContext().tunnelManager().selectInboundTunnel();
+            _replyTunnel = ctx.tunnelManager().selectInboundTunnel();
             _outTunnel = _cfg;
             _otherTunnel = (PooledTunnelCreatorConfig) _replyTunnel;
         }
@@ -92,61 +93,59 @@ class TestJob extends JobImpl {
         if ( (_replyTunnel == null) || (_outTunnel == null) ) {
             if (_log.shouldLog(Log.WARN))
                 _log.warn("Insufficient tunnels to test " + _cfg + " with: " + _replyTunnel + " / " + _outTunnel);
-            getContext().statManager().addRateData("tunnel.testAborted", _cfg.getLength(), 0);
+            ctx.statManager().addRateData("tunnel.testAborted", _cfg.getLength());
             scheduleRetest();
         } else {
             int testPeriod = getTestPeriod();
-            long testExpiration = getContext().clock().now() + testPeriod;
-            DeliveryStatusMessage m = new DeliveryStatusMessage(getContext());
-            m.setArrival(getContext().clock().now());
+            long now = ctx.clock().now();
+            long testExpiration = now + testPeriod;
+            DeliveryStatusMessage m = new DeliveryStatusMessage(ctx);
+            m.setArrival(now);
             m.setMessageExpiration(testExpiration);
-            m.setMessageId(getContext().random().nextLong(I2NPMessage.MAX_ID_VALUE));
-            ReplySelector sel = new ReplySelector(getContext(), m.getMessageId(), testExpiration);
-            OnTestReply onReply = new OnTestReply(getContext());
-            OnTestTimeout onTimeout = new OnTestTimeout(getContext());
-            OutNetMessage msg = getContext().messageRegistry().registerPending(sel, onReply, onTimeout);
+            m.setMessageId(ctx.random().nextLong(I2NPMessage.MAX_ID_VALUE));
+            ReplySelector sel = new ReplySelector(m.getMessageId(), testExpiration);
+            OnTestReply onReply = new OnTestReply();
+            OnTestTimeout onTimeout = new OnTestTimeout();
+            OutNetMessage msg = ctx.messageRegistry().registerPending(sel, onReply, onTimeout);
             onReply.setSentMessage(msg);
-            sendTest(m);
+            sendTest(m, testPeriod);
         }
     }
     
-    private void sendTest(I2NPMessage m) {
+    private void sendTest(I2NPMessage m, int testPeriod) {
         // garlic route that DeliveryStatusMessage to ourselves so the endpoints and gateways
         // can't tell its a test.  to simplify this, we encrypt it with a random key and tag,
         // remembering that key+tag so that we can decrypt it later.  this means we can do the
         // garlic encryption without any ElGamal (yay)
-        PayloadGarlicConfig payload = new PayloadGarlicConfig(Certificate.NULL_CERT,
-                                                              getContext().random().nextLong(I2NPMessage.MAX_ID_VALUE),
-                                                              m.getMessageExpiration(),
-                                                              DeliveryInstructions.LOCAL, m);
-        payload.setRecipient(getContext().router().getRouterInfo());
-
-        SessionKey encryptKey = getContext().keyGenerator().generateSessionKey();
-        SessionTag encryptTag = new SessionTag(true);
-        _encryptTag = encryptTag;
-        Set<SessionTag> sentTags = null;
-        GarlicMessage msg = GarlicMessageBuilder.buildMessage(getContext(), payload, sentTags, 
-                                                              getContext().keyManager().getPublicKey(), 
-                                                              encryptKey, encryptTag);
-
+        final RouterContext ctx = getContext();
+        MessageWrapper.OneTimeSession sess;
+        if (_cfg.isInbound() && !_pool.getSettings().isExploratory()) {
+            // to client. false means don't force AES
+            sess = MessageWrapper.generateSession(ctx, _pool.getSettings().getDestination(), testPeriod, false);
+        } else {
+            // to router. AES.
+            sess = MessageWrapper.generateSession(ctx, testPeriod);
+        }
+        if (sess == null) {
+            scheduleRetest();
+            return;
+        }
+        // null for ratchet
+        _encryptTag = sess.tag;
+        GarlicMessage msg;
+        if (sess.tag != null) // AES
+            msg = MessageWrapper.wrap(ctx, m, sess.key, sess.tag);
+        else // ratchet
+            msg = MessageWrapper.wrap(ctx, m, sess.key, sess.rtag);
         if (msg == null) {
             // overloaded / unknown peers / etc
             scheduleRetest();
             return;
         }
-        Set<SessionTag> encryptTags = new RemovableSingletonSet<SessionTag>(encryptTag);
-        // Register the single tag with the appropriate SKM
-        if (_cfg.isInbound() && !_pool.getSettings().isExploratory()) {
-            SessionKeyManager skm = getContext().clientManager().getClientSessionKeyManager(_pool.getSettings().getDestination());
-            if (skm != null)
-                skm.tagsReceived(encryptKey, encryptTags);
-        } else {
-            getContext().sessionKeyManager().tagsReceived(encryptKey, encryptTags);
-        }
-
+        _id = __id.getAndIncrement();
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Sending garlic test of " + _outTunnel + " / " + _replyTunnel);
-        getContext().tunnelDispatcher().dispatchOutbound(msg, _outTunnel.getSendTunnelId(0),
+            _log.debug("Sending garlic test #" + _id + " of " + _outTunnel + " / " + _replyTunnel);
+        ctx.tunnelDispatcher().dispatchOutbound(msg, _outTunnel.getSendTunnelId(0),
                                                          _replyTunnel.getReceiveTunnelId(0),
                                                          _replyTunnel.getPeer(0));
     }
@@ -154,8 +153,8 @@ class TestJob extends JobImpl {
     public void testSuccessful(int ms) {
         if (_pool == null || !_pool.isAlive())
             return;
-        getContext().statManager().addRateData("tunnel.testSuccessLength", _cfg.getLength(), 0);
-        getContext().statManager().addRateData("tunnel.testSuccessTime", ms, 0);
+        getContext().statManager().addRateData("tunnel.testSuccessLength", _cfg.getLength());
+        getContext().statManager().addRateData("tunnel.testSuccessTime", ms);
     
         _outTunnel.incrementVerifiedBytesTransferred(1024);
         // reply tunnel is marked in the inboundEndpointProcessor
@@ -170,7 +169,7 @@ class TestJob extends JobImpl {
             _otherTunnel.testJobSuccessful(ms);
 
         if (_log.shouldLog(Log.DEBUG))
-            _log.debug("Tunnel test successful in " + ms + "ms: " + _cfg);
+            _log.debug("Tunnel test #" + _id + " successful in " + ms + "ms: " + _cfg);
         scheduleRetest();
     }
     
@@ -193,7 +192,7 @@ class TestJob extends JobImpl {
         else
             getContext().statManager().addRateData("tunnel.testFailedTime", timeToFail, timeToFail);
         if (_log.shouldLog(Log.WARN))
-            _log.warn("Tunnel test failed in " + timeToFail + "ms: " + _cfg);
+            _log.warn("Tunnel test #" + _id + " failed in " + timeToFail + "ms: " + _cfg);
         boolean keepGoing = _cfg.tunnelFailed();
         // blame the expl. tunnel too
         if (_otherTunnel.getLength() > 1)
@@ -234,6 +233,7 @@ class TestJob extends JobImpl {
     }
 
     private void scheduleRetest() { scheduleRetest(false); }
+
     private void scheduleRetest(boolean asap) {
         if (_pool == null || !_pool.isAlive())
             return;
@@ -248,18 +248,16 @@ class TestJob extends JobImpl {
     }
     
     private class ReplySelector implements MessageSelector {
-        private final RouterContext _context;
         private final long _id;
         private final long _expiration;
 
-        public ReplySelector(RouterContext ctx, long id, long expiration) {
-            _context = ctx;
+        public ReplySelector(long id, long expiration) {
             _id = id;
             _expiration = expiration;
             _found = false;
         }
         
-        public boolean continueMatching() { return !_found && _context.clock().now() < _expiration; }
+        public boolean continueMatching() { return !_found && getContext().clock().now() < _expiration; }
 
         public long getExpiration() { return _expiration; }
 
@@ -286,7 +284,7 @@ class TestJob extends JobImpl {
         private long _successTime;
         private OutNetMessage _sentMessage;
 
-        public OnTestReply(RouterContext ctx) { super(ctx); }
+        public OnTestReply() { super(TestJob.this.getContext()); }
 
         public String getName() { return "Tunnel test success"; }
 
@@ -322,22 +320,23 @@ class TestJob extends JobImpl {
     private class OnTestTimeout extends JobImpl {
         private final long _started;
 
-        public OnTestTimeout(RouterContext ctx) { 
-            super(ctx); 
-            _started = ctx.clock().now();
+        public OnTestTimeout() { 
+            super(TestJob.this.getContext()); 
+            _started = getContext().clock().now();
         }
 
         public String getName() { return "Tunnel test timeout"; }
 
         public void runJob() {
             if (_log.shouldLog(Log.WARN))
-                _log.warn("Timeout: found? " + _found);
+                _log.warn("Tunnel test #" + _id + " timeout: found? " + _found);
             if (!_found) {
                 // don't clog up the SKM with old one-tag tagsets
                 if (_cfg.isInbound() && !_pool.getSettings().isExploratory()) {
                     SessionKeyManager skm = getContext().clientManager().getClientSessionKeyManager(_pool.getSettings().getDestination());
-                    if (skm != null)
+                    if (skm != null && _encryptTag != null)
                         skm.consumeTag(_encryptTag);
+                    // else null tag for ratchet, let it expire
                 } else {
                     getContext().sessionKeyManager().consumeTag(_encryptTag);
                 }
